Installation
=====

#### Prerequisites

- Linux Server (Tested on Red Hat 8 and Ubuntu 18.04+)
- Root permission access for installation
- Dynatrace API Token with applicable permissions. See [Security](/README/Security).
- Installing as an extension on a Dynatrace [ActiveGate](https://www.dynatrace.com/support/help/shortlink/activegate-hub)
  - ActiveGate server runs on Linux
  - ActiveGate has `Extensions 1.0` and `REST API` modules enabled


#### Installing the Dynatrace Solution Server (DSS) on an ActiveGate
- Go to the [Dynatrace ESA Solution Server](https://www.dynatrace.com/hub/detail/dynatrace-solution-server-by-esa/) on [Dynatrace Hub](https://www.dynatrace.com/hub). The latest Solution Server is available via the "Download" button.
- SSH onto the ActiveGate that you want to install the Solution Server, using a user that has access to the ActiveGate installation. 
- Download the solution archive into `/opt/dynatrace/remotepluginmodule/plugin_deployment/` by:
  - Right-clicking and copy the link address from the "Download" button and download with `curl` or `wget`
  - OR downloading to your local machine and transferring by your preferred means
- Rename the file if downloaded with a random suffix, such as `custom.remote.python.esa-solu_3fpMyA2.zip`
  - `mv custom.remote.python.esa-solu_3fpMyA2.zip custom.remote.python.esa-solution-server-v1.zip`
- Decompress the file. 
  - `unzip -o ./custom.remote.python.esa-solution-server-v1.zip`
- You now should see the following in the `plugin_deployment` directory:
  - `custom.remote.python.esa-solution-server-v1`
  - `custom.remote.python.esa-solution-server-v1.zip`
  - You may now delete the `custom.remote.python.esa-solution-server-v1.zip` file.
- In the `/opt/dynatrace/remotepluginmodule/plugin_deployment/custom.remote.python.esa-solution-server-v1` directory:
  - Run the command `./dss install` as `root` 
  - Configure the DSS for first-time startup with `dss config`
  - Start the DSS with `dss launch` as the user that should own the process.
  - For further details, refer to [Security](/README/Security).
- Navigate to the Solution Server via your browser at `https://your-solution-server.corp.com:443`

#### Installing the DSS standalone
- Using the procedure above, instead of using the `/opt/dynatrace/remotepluginmodule/plugin_deployment/` directory, you may specify a directory of your choice. On Linux, we recommend using the directory `/opt/dynatrace/esa/solution-server/`. Note that any Solutions you install later using the DSS will be installed into a subdirectory of the parent, e.g. `/opt/dynatrace/esa/my-new-solution/`.

Last Steps
=====
At this point, the DSS should be installed and running. By default it will generate a self signed certificate for HTTPS termination. You can navigate the UI e.g. `https://my-ag.corp/` to apply your own certificate and configure the authentication mechanism used by the DSS. By default, the DSS will use Dynatrace tokens that test against a local ActiveGate on port `9999`. This behavior can be overridden in the `config/authorizations.json` file.

## Updating

```bash
# Save the current process list
dss save
# Stop the running DSS Processes for safe upgrade.
dss kill
# Unzip the new archive
unzip -o ./custom.remote.python.esa-solution-server-v1.zip
# Create and update links in PATH; create service.
dss install
# Run the DSS
dss launch
```


## DSS CLI
| command | action |
| --- | --- |
| dss start   | Starts the DSS program |
| dss stop    | Stops the DSS program |
| dss launch  | Similar to the Start command, starts the program and saves the current process state for reboot |
| dss install | Installs the binary dependencies into the system and sets file capabilities |
| dss config  | Opens up a configuration screen to reset your base Authentication settings |
| dss restart | Immediately kills all DSS processes and restarts them |
| dss reload  | Rolling-restart kills DSS processes slowly and brings new ones back online |
| dss <args>  | Will act as PM2 arguments passed through |


