Security
=====

What mechanisms are in place to keep the Solution Server secure?

Frontend
 - All communication is secured with HTTPS using HSTS.
 - Only known script and style domains may execute frontend code.
 - Authentication credentials are stored in the named `SessionStorage` object. They are disposed on session termination.
 - Reading configuration, writing configuration, reading logs & start/stop/restarting of solutions requires permissions.
 - Authentication credentials are passed in an encoded `Authorization` header.

Backend
 - All backend processing that may make changes requires authorization.
 - If authorization mechanism is unavailable or otherwise fails, the action will be rejected.
 - Authorization failures ratelimit to 2 requests per minute by IP address. This makes it such that a "Brute force" attempt to discover a username/password credential pair would be impractical

# Authentication Mechanisms

- Dynatrace API Authentication
  - You enter a Dynatrace API token that is tested when authorization is required.
  - The token must be valid, not revoked, and provide sufficient access scopes for the requested action.
- Users List
  - You create a username and password & assign "read config" "write config" and or "read logs" permissions.
- SAML / OpenID
  

# Roles
Upon clicking any button that requires permissions, a modal popup is shown. Tokens or credentials entered are stored as transient session cookies in the browser. 

**Permissions Required**  

## Using Dynatrace API Authorization
This functionality requires a Dynatrace token with the following permissions: 

#### The `Settings` page 
| Scope | Purpose |
| ---  | --- |
| `REQ` Read config  | To see the settings |
| `OPT` Write config | To edit settings.   |
| (more…) |

#### The `Health` page 
| Scope | Purpose |
| ---  | --- |
| `OPT` Read logs    | To view log files. |
| `OPT` Read metrics | To view advanced health metrics. |
| (more…) |

#### The `Install` wizard 
| Scope | Purpose |
| ---  | --- |
| `REQ` Read config     | The install and configure the extension.  |
| `REQ` Write config    | The install and configure the extension.  |
| `REQ` Activegate read | To register the extension with Dynatrace. |
| (more…) |

#### Total permissions required: 
| Scope | Purpose |
| ---  | --- |
| Read config     | To see the settings. |
| Write config    | To edit settings & install extensions. |
| Read logs 	  | To view log files. |
| Read metrics    | To view advanced health metrics. |
| Activegate read | To register the extension with Dynatrace. |

## Using User based Authorization
In the user authorization flow, users have a `username` and `password`. Each user is created with optional roles specified. 

#### Permissions
| Scope | Purpose |
| ---  | --- |
| `Read Logs`           | to Read log content through the DSS UI |
| `Read Configuration`  | To Read configuration |
| `Write Configuration` | To Save configuration changes |


# Filesystem Permissions (Linux)

When setting up the DSS, the following conditions must be met:

 - It must run as a user that can listen on port `443`. (or at the port specified in the config)
 - The directory should be owned by the same user that runs the executable:
     ```shell
     drwxr-xr-x 12 myUser myGroup 4096 Jan  4 20:56 ./
     drwxr-xr-x  3 myUser myGroup 4096 Jan  4 20:56 ../
     -rw-rw-r--  1 myUser myGroup 4486 Jan  4 15:55 README.md
     drwxr-xr-x  5 myUser myGroup 4096 Jan  4 20:56 client/
     drwxr-xr-x  2 myUser myGroup 4096 Jan  4 20:56 config/
     drwxr-xr-x  2 myUser myGroup 4096 Dec  6 18:57 data/
     drwxr-xr-x  5 myUser myGroup 4096 Jan  4 20:56 doc/
     -rwxrwxr-x  1 myUser myGroup  296 Jan  4 15:55 dss*
     -rw-rw-r--  1 myUser myGroup  411 Jan  4 15:55 ecosystem.config.js
     drwxr-xr-x  2 myUser myGroup 4096 Dec  6 13:05 examples/
     drwxr-xr-x  2 myUser myGroup 4096 Jan  4 20:47 log/
     drwxr-xr-x  6 myUser myGroup 4096 Dec  6 13:54 node/
     -rw-rw-r--  1 myUser myGroup 3417 Jan  4 15:55 plugin.json
     -rw-rw-r--  1 myUser myGroup 1809 Jan  4 15:55 plugin.py
     drwxr-xr-x 17 myUser myGroup 4096 Jan  4 20:56 pylib/
     drwxr-xr-x  7 myUser myGroup 4096 Jan  4 20:56 server/
     drwxr-xr-x  2 myUser myGroup 4096 Dec  6 18:57 tmp/
     ```

 - In order to use the Solution Server to install solutions, the user that starts the DSS needs to be able to read/write the parent directory: 
   ```shell
   drwxr-xr-x  3 myUser root    4096 Jan  4 22:05 ./
   drwxr-xr-x  4 root   root    4096 Dec 17 15:35 ../
   drwxr-xr-x 12 myUser myGroup 4096 Jan  4 20:56 custom.remote.python.esa-solution-server-v1/
   ```

 - The processes will be owned by the user that started the solution server. (`./dss launch`)
 - The following files must be marked executable: (`./dss install` does this for you)
   - `$dir/node/bin/node`
   - `$dir/server/node_modules/pm2/bin/pm2`






