"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const pm2_1 = require("../utils/pm2");
const express_1 = __importDefault(require("express"));
const utils_1 = require("../utils/utils");
const authorizer_1 = require("@dt-esa/authorizer");
const errorhandler_1 = require("../utils/errorhandler");
const http_errors_1 = __importDefault(require("http-errors"));
const globals_1 = require("../globals");
const AnsiConverter = require('ansi-to-html');
const ansiConvert = new AnsiConverter();
const add = (x, y) => x + y;
const router = express_1.default.Router();
// Solution start/stop/etc.
router.get('/:id/:action', (0, authorizer_1.authorize)(["ReadConfig", "WriteConfig"]), (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const id = req.params.id;
    let solution = globals_1.solutionFinder.findSolutionById(id);
    if (!solution) {
        yield globals_1.solutionFinder.discover();
        solution = globals_1.solutionFinder.findSolutionById(id);
        if (!solution)
            return res.status(400).send("Solution not found");
    }
    let result = null;
    try {
        switch (req.params.action) {
            case "delete":
                result = yield (0, pm2_1.del)(solution.config.id);
                break;
            case "refresh":
                result = yield (0, pm2_1.del)(solution.config.id);
                result = yield (0, pm2_1.startSolution)(solution);
                break;
            case "start":
                result = yield (0, pm2_1.startSolution)(solution);
                break;
            case "restart":
                result = yield (0, pm2_1.restart)(solution.config.id);
                break;
            case "reload":
                if (globals_1.solutionFinder.requiresRestart(solution))
                    result = yield (0, pm2_1.restart)(solution.config.id);
                else
                    result = yield (0, pm2_1.reload)(solution.config.id);
                break;
            case "stop":
                result = yield (0, pm2_1.stop)(solution.config.id);
                break;
        }
    }
    catch (ex) {
        return res.status(500).send("The " + req.params.action + " action failed " + (Array.isArray(ex) ? ex.join('\n') :
            ex.message ? ":" + ex.message : ""));
    }
    // TODO: The status that we are sending back here is actually a bit richer
    // than what we get with describeApps. The status of the process is here.
    // We need to use that better in the UI.
    return res.send(result);
})));
// Method to restart a single instance.
router.get('/:id/:instance/:action', (0, authorizer_1.authorize)(["ReadConfig", "WriteConfig"]), (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const id = req.params.id;
    let solution = globals_1.solutionFinder.findSolutionById(id);
    if (!solution) {
        yield globals_1.solutionFinder.discover();
        solution = globals_1.solutionFinder.findSolutionById(id);
        if (!solution)
            return res.status(400).send("Solution not found");
    }
    const instance = req.params.instance;
    let result = null;
    try {
        switch (req.params.action) {
            case "delete":
                result = yield (0, pm2_1.del)(instance);
                break;
            case "restart":
                result = yield (0, pm2_1.restart)(instance);
                break;
            case "reload":
                result = yield (0, pm2_1.reload)(instance);
                break;
            case "stop":
                result = yield (0, pm2_1.stop)(instance);
                break;
        }
    }
    catch (ex) {
        return res.status(500).send("The " + req.params.action + " action failed " + (Array.isArray(ex) ? ex.join('\n') :
            ex.message ? ":" + ex.message : ""));
    }
    return res.send(result);
})));
// Health status
router.get('/:id', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const id = req.params.id;
    let solution = globals_1.solutionFinder.findSolutionById(id);
    if (!solution) {
        yield globals_1.solutionFinder.discover();
        solution = globals_1.solutionFinder.findSolutionById(id);
        if (!solution)
            return res.status(400).send("Solution not found");
    }
    if (!solution.config) {
        debugger;
        throw new Error("No solution config for " + id);
    }
    // return res.send(await list());
    // TODO: Create a type for an ecosystem app.
    const daemonRunApps = yield (0, pm2_1.describe)(solution.config.id);
    const ecosystemApps = globals_1.solutionFinder.getEcosystemApps(solution.config.id);
    const appNames = daemonRunApps.map(app => app.name);
    const plannedApps = ecosystemApps
        .filter(app => !appNames.includes(app.name))
        .map(app => ({
        name: app.name,
        pid: 0,
        pm_id: 0,
        uptime: 0,
        monit: { memory: 0, cpu: 0 },
        pm2_env: { status: 'one-launch-status' }
    }));
    const runningApps = daemonRunApps.length > 0 ? daemonRunApps.filter(app => app.pm2_env.status === "online") : [];
    const isRunning = runningApps.length > 0 && runningApps.length === daemonRunApps.length;
    let sol = {
        name: solution.name,
        id: solution.config.id,
        namespace: solution.config.name,
        pluginName: solution.config.pluginName,
        title: solution.config.title,
        path: solution.config.path,
        mode: solution.config.mode,
        image: solution.config.image,
        version: solution.config.version,
        port: solution.config.port || "N/A",
        protocol: solution.config.protocol || "N/A",
        status: isRunning ? "running" : "stopped",
        totalCPU: daemonRunApps.length > 0 ? daemonRunApps.map(app => app.monit.cpu).reduce(add, 0) : 0,
        totalMem: daemonRunApps.length > 0 ? daemonRunApps.map(app => app.monit.memory).reduce(add, 0) : 0
    };
    /*    if (deamonRunApps.length > 0) {
            //let mainApp = apps.filter(app => app.pm2_env['NODE_APP_INSTANCE'] === 0);
             let logs = {};
            
            try {
                let stdout: any = await readLogs({file_path: mainApp.pm2_env.pm_out_log_path});
                let stderr: any = await readLogs({file_path: mainApp.pm2_env.pm_err_log_path});
                stdout.lines = stdout.lines.map(log => ansiConvert.toHtml(log)).join('<br/>');
                stderr.lines = stderr.lines.map(log => ansiConvert.toHtml(log)).join('<br/>');
                logs = { stdout, stderr };
            }
            catch(ex) {
                console.log(ex);
            }
    
            // The mainApp *should* be the master.
            // The apps should be all the instances.
            // Logs should be only from the master instance.
        }*/
    return res.send({ sol: sol, apps: [...daemonRunApps, ...plannedApps] });
})));
router.use((req, res, next) => next((0, http_errors_1.default)(404)));
router.use(errorhandler_1.ErrorHandler);
module.exports = router;
