"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const utils_1 = require("../utils/utils");
const globals_1 = require("../globals");
const router = express_1.default.Router();
// Get All Discovered Solutions and Extensions.
router.get('/api/modules', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const moduleConfigs = globals_1.solutionFinder
        .getAll()
        .map(mod => mod.config);
    res.send(moduleConfigs);
})));
// Return JSON for health metrics back to Dynatrace.
router.get('/api/health', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const solutions = globals_1.solutionFinder.getSolutions();
    const extensions = globals_1.solutionFinder.getExtensions();
    res.send({
        pluginStackSize: solutions.map(sol => sol.router.stack).filter(p => p).reduce((total, router) => total + router.length, 0),
        runningPlugins: solutions.filter(sol => sol.config.status == 'ok').length - 1,
        crashedPlugins: solutions.filter(sol => sol.config.status == 'failedStart').length,
        awaitingConfigPlugins: solutions.filter(sol => sol.config.status == 'configRequired').length,
        extensions: extensions.length
    });
})));
// Get any configuration that's relevant to the UI and isn't a security risk.
router.get('/api/config', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const auth = globals_1.AuthConfig;
    res.send({
        uiLoginMode: auth.uiLoginMode || "dynatrace" // "dynatrace" "saml" "openid"
    });
})));
module.exports = router;
