"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const shelljs_1 = __importDefault(require("shelljs"));
const utils_1 = require("../utils/utils");
const fs = __importStar(require("fs"));
const adm_zip_1 = __importDefault(require("adm-zip"));
const path_1 = __importDefault(require("path"));
const formidable_1 = __importDefault(require("formidable"));
const form_data_1 = __importDefault(require("form-data"));
const chalk_1 = __importDefault(require("chalk"));
const globals_1 = require("../globals");
let router = express_1.default.Router();
const stageDir = path_1.default.normalize(__dirname + `/../../tmp/`);
// Unzip the plugin in the correct directory.
// Will overwrite dir contents -- useful for updating the plugin.
function installPlugin(zipName, req) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            let localTempDir = stageDir + zipName;
            globals_1.repoLogger.info("Loading '" + zipName + ".zip'");
            let zip = new adm_zip_1.default(localTempDir + '.zip');
            // Extract all of the contents.
            globals_1.repoLogger.info(`Unzipping '${zipName}' to ${localTempDir}`);
            zip.extractAllTo(localTempDir + '/', true); // overwrite true
            globals_1.repoLogger.info(`Finished unzipping '${zipName}'`);
            let pluginDirs = (0, utils_1.getDirectories)(localTempDir + '/');
            const dssFile = localTempDir + '/' + pluginDirs[0] + '/dss.config.json';
            const dssJson = fs.existsSync(dssFile) ? JSON.parse(fs.readFileSync(dssFile, { encoding: "utf-8" }).trim()) : {};
            let hintMessage = "";
            // If we don't have a dss.config.json
            if (Object.keys(dssJson).length == 0) {
                // Check for any files that would suggest this is a Solution, not an extension.
                // hintFiles.forEach(hf => {
                //     if (fs.existsSync(localTempDir + "/" + pluginDirs[0] + "/" + hf))
                //         solutionHint = true;
                // });
                const hintFiles = ["ecosystem.config.js", "ecosystem.config.json", "package.json", "client", "server"];
                const solutionHint = hintFiles.some(file => fs.existsSync(localTempDir + "/" + pluginDirs[0] + "/" + file));
                if (solutionHint) {
                    // We are probably an extension, but dss.config.json is missing.
                    hintMessage = `    We installed this as an extension, but similarities to a Solution were detected. If this is unintended, check if '/dss.config.json' exists in the ZIP file.`;
                    globals_1.repoLogger.warn(hintMessage);
                    console.warn(hintMessage);
                }
                else {
                    // This really looks like an Extension.
                }
                const src = localTempDir + '/';
                const target = path_1.default.normalize(localTempDir + '/../../../');
                // For each directory in the installed ZIP, clone it to the installation directory.
                let dirItems = (0, utils_1.getDirectories)(localTempDir);
                const pluginFile = localTempDir + '/' + pluginDirs[0] + '/plugin.json';
                const pluginJson = fs.existsSync(pluginFile) ? JSON.parse(fs.readFileSync(pluginFile, { encoding: "utf-8" }).trim()) : {};
                dirItems.forEach(item => {
                    shelljs_1.default.mv('-f', src + "/" + item, target + "/");
                });
                return {
                    id: dirItems[0].split("/").pop(),
                    installedAsExtension: true,
                    hintMessage,
                    directory: dirItems[0],
                    json: pluginJson
                };
            }
            return {
                dssFile,
                localTempDir,
                zipName,
                hintMessage,
                instanceName: "default",
                portNumber: dssJson.port,
                json: JSON.stringify(dssJson)
            };
        }
        catch (ex) {
            return ex;
        }
    });
}
// Move from ~/tmp/ and install it in the correct target directory.
// TODO: Handle plugin JSON ids...?
function unstageAndInstall({ localTempDir, zipName, instanceName, portNumber }, req) {
    return __awaiter(this, void 0, void 0, function* () {
        let uploadZipResult;
        let endpointResult;
        try {
            let pluginDirs = (0, utils_1.getDirectories)(localTempDir + '/').concat((0, utils_1.getFiles)(localTempDir + '/'));
            globals_1.repoLogger.info(`Writing '${zipName}' dss.config.json...`);
            const currentSolutionDir = pluginDirs.filter(d => !/solution-server/.test(d))[0];
            // Create instance dss.config.json
            const dssFile = localTempDir + '/' + currentSolutionDir + '/dss.config.json';
            const dssJson = fs.existsSync(dssFile) ? JSON.parse(fs.readFileSync(dssFile, { encoding: "utf-8" }).trim()) : {};
            const instanceSlug = instanceName.replace(/[^a-zA-Z0-9\-_]/g, '');
            const isDefaultInstance = instanceName == "default" || instanceName == "";
            globals_1.repoLogger.info(`Created slug '${instanceSlug}'.`);
            dssJson.port = portNumber;
            // If an instance was specified, modify the path and title.
            if (isDefaultInstance == false) {
                dssJson.title = dssJson.title.split('[')[0] + ' [' + instanceName + ']';
                dssJson.path = '/' + dssJson.path.split('/')[1] + '-' + instanceSlug.toLowerCase();
            }
            fs.writeFileSync(dssFile, JSON.stringify(dssJson, null, 4));
            globals_1.repoLogger.info(`...Done writing dss.config.json for  '${zipName}'.`);
            globals_1.repoLogger.info(`Uploading '${zipName}' to Dynatrace...`);
            uploadZipResult = yield uploadZipToDynatrace(zipName, req);
            globals_1.repoLogger.info(`...Finished uploading '${zipName}' to Dynatrace.`);
            // TODO: handle uploading the same plugin twice.
            // If we were able to upload the zip without issues.
            if (uploadZipResult.success == true) {
                globals_1.repoLogger.info(`Creating endpoint for '${zipName}'...`);
                endpointResult = yield createDynatraceEndpoint(zipName, req);
                globals_1.repoLogger.info(`...Finished creating endpoint for '${zipName}'.`);
            }
            // Move the unzipped contents to the plugin dir.
            // TODO: Here, we should allow for a postfix to distinguish
            // this installation from others with the same ID.
            // Example: esa-etl-service-walmart, esa-etl-service-jpmc.
            globals_1.repoLogger.info(`Deploying '${zipName}'...`);
            // Get the DIR name of the plugin.
            let solutions = [];
            for (let i = 0; i < pluginDirs.length; i++) {
                const src = localTempDir + '/' + pluginDirs[i] + '/*';
                const target = localTempDir + '/../../../' + pluginDirs[i] + '-' + instanceSlug;
                if (!fs.existsSync(target))
                    fs.mkdirSync(target);
                // Clone to output
                shelljs_1.default.cp('-R', src, target);
                globals_1.repoLogger.info("Copying " + src + ' -> ' + target);
                // Analyze plugin for next UI steps.
                let pluginDirName = (0, utils_1.getDirectories)(localTempDir + '/');
                let pluginName = pluginDirName[0];
                const serverFile = `${localTempDir}/../../../${pluginName}/server/package.json`;
                const packageFile = `${localTempDir}/../../../${pluginName}/package.json`;
                const pluginFile = `${localTempDir}/../../../${pluginName}/plugin.json`;
                const configFile = `${localTempDir}/../../../${pluginName}/dss.config.json`;
                const serverJson = fs.existsSync(serverFile) ? JSON.parse(fs.readFileSync(serverFile, { encoding: "utf-8" }).trim()) : {};
                const packageJson = fs.existsSync(packageFile) ? JSON.parse(fs.readFileSync(packageFile, { encoding: "utf-8" }).trim()) : {};
                const pluginJson = fs.existsSync(pluginFile) ? JSON.parse(fs.readFileSync(pluginFile, { encoding: "utf-8" }).trim()) : {};
                const configJson = fs.existsSync(configFile) ? JSON.parse(fs.readFileSync(configFile, { encoding: "utf-8" }).trim()) : {};
                dssJson.version = pluginJson.version || dssJson.version || "0.0.1";
                solutions.push({
                    // It is a solution if either of the following are satisfied.
                    id: pluginDirs[i] + '-' + instanceSlug,
                    isSolution: !!configFile || !!serverJson["esa-solution-server"] || !!packageJson["esa-solution-server"],
                    config: configFile || serverJson["esa-solution-server"] || packageJson["esa-solution-server"],
                    pluginJson
                });
            }
            yield globals_1.solutionFinder.discover();
            // Get the whole solution data as would be seen by the discovery process.
            solutions = solutions.map(s => {
                s.config = globals_1.solutionFinder.findSolutionById(s.id) || globals_1.solutionFinder.findExtensionById(s.id) || s.config;
                return s;
            });
            // TODO: Uncomment
            // Remove the temporary files.
            // log.info(`Cleaning up after '${zipName}'...`);
            // shell.rm('-rf', `${localTempDir}/`);
            // shell.rm('-f', `${localTempDir}.zip`);
            // log.info(`Cleaned up after '${zipName}'.`);
            return {
                id: solutions[0].id,
                hintMessage: "You can visit the Settings and Health pages for this Solution from the tile that has been added.",
                directory: solutions[0].id,
                json: dssJson,
                success: uploadZipResult,
                endpointResult,
                solutions: solutions
            };
        }
        catch (ex) {
            // Collate results into a better exception. We still want to reject with a 500
            // so we throw a new exception.
            globals_1.repoLogger.error("Could not fully complete installation");
            globals_1.repoLogger.error("");
            globals_1.repoLogger.error("Message: " + ex.message);
            globals_1.repoLogger.error("Stack: \n" + ex.stack);
            globals_1.repoLogger.error("Upload result:");
            globals_1.repoLogger.error("uploadZipResult::" + (uploadZipResult || "None"));
            globals_1.repoLogger.error("Endpoint creation result:");
            globals_1.repoLogger.error("endpointResult::" + (endpointResult || "None"));
            console.log(chalk_1.default.redBright(ex), uploadZipResult, endpointResult);
            throw new Error(JSON.stringify({
                success: false,
                ex,
                uploadZipResult,
                endpointResult
            }, null, 4));
        }
    });
}
function uploadZipToDynatrace(dir, req) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    return __awaiter(this, void 0, void 0, function* () {
        const dtUrl = globals_1.Config.dtUrl;
        try {
            let pluginDirName = (0, utils_1.getDirectories)(stageDir + dir + '/');
            let pluginName = pluginDirName[0];
            let pluginFile = `${stageDir}${dir}/${pluginName}/plugin.json`;
            let zipFile = stageDir + `${pluginName}_ui.zip`;
            if (!fs.existsSync(pluginFile))
                return { message: "No plugin.json file found" };
            globals_1.repoLogger.info(`Zipping plugin.json`);
            let zip = new adm_zip_1.default();
            zip.addLocalFile(pluginFile);
            zip.writeZip(zipFile);
            globals_1.repoLogger.info(`Zipped plugin.json`);
            const formData = new form_data_1.default();
            formData.append('file', fs.createReadStream(zipFile), {
                filename: pluginName + '.zip'
            });
            let headers = formData.getHeaders();
            headers['Authorization'] = "Api-Token " + (req.header('Authorization') || globals_1.AuthConfig.altToken);
            let response;
            try {
                response = yield globals_1.axios.post(`${dtUrl}api/config/v1/extensions`, formData, {
                    headers
                });
                globals_1.repoLogger.info(`Successfully uploaded extension zip`);
                return {
                    success: true,
                    message: "success",
                    data: response.data
                };
            }
            catch (ex) {
                // Handle this instance specially.
                if (((_b = (_a = ex.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.constraintViolations[0].message) == 'new uploaded extension must not have the same version as stored binary.') {
                    return { success: true, message: "Plugin already present." };
                }
                globals_1.repoLogger.error(`Failed to upload extension zip`);
                let message = `${(_d = (_c = ex.response) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.code} : ${(_f = (_e = ex.response) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.message} : ${JSON.stringify((_h = (_g = ex.response) === null || _g === void 0 ? void 0 : _g.data) === null || _h === void 0 ? void 0 : _h.constraintViolations)}`;
                console.log(chalk_1.default.redBright(message));
                globals_1.repoLogger.error(message);
                globals_1.repoLogger.error(ex.stack || ex.message || ex.toString());
                return {
                    message: ex.response.data.message || ex.message || 'Failure',
                    ex
                };
            }
        }
        catch (ex) {
            return ex;
        }
    });
}
function createDynatraceEndpoint(dir, req) {
    return __awaiter(this, void 0, void 0, function* () {
        const dtUrl = globals_1.Config.dtUrl;
        let pluginId;
        let payload;
        try {
            const pluginDir = (0, utils_1.getDirectories)(stageDir + dir + '/');
            if (pluginDir.length > 1)
                globals_1.repoLogger.warn("Multiple folders detected. Attempting to create endpoint for: " + pluginDir[0]);
            const pluginFile = `${stageDir}${dir}/${pluginDir[0]}/plugin.json`;
            const pluginConfig = fs.existsSync(pluginFile) ? JSON.parse(fs.readFileSync(pluginFile, { encoding: 'utf8' }).trim()) : {};
            pluginId = pluginConfig.name;
            let { data } = yield globals_1.axios.get(`${dtUrl}api/config/v1/extensions/activeGateExtensionModules`, {
                headers: {
                    Authorization: "Api-Token " + req.header('Authorization')
                }
            });
            const hostname = shelljs_1.default.exec('hostname').replace(/[\r\n]/g, '');
            const rpmId = data.values.find(v => v.name.toLowerCase() == hostname.toLowerCase()).id;
            let properties = {};
            pluginConfig.properties.forEach(prop => {
                properties[prop.key] =
                    pluginConfig.configUI.properties.find(p => p.key == prop.key).defaultValue ||
                        (prop.type == "String" && "") || 0;
            });
            payload = {
                // ref. https://www.dynatrace.com/support/help/shortlink/api-config-extensions-instances-post-new#body-format
                id: pluginId,
                enabled: true,
                useGlobal: true,
                endpointName: hostname + ' ' + new Date().getTime().toString(32),
                properties,
                activeGate: {
                    id: rpmId
                }
            };
            console.log("Creating endpoint", `${dtUrl}api/config/v1/extensions/${pluginId}/instances`, payload);
        }
        catch (ex) {
            globals_1.repoLogger.error(ex.response.data || ex);
            return ex.response.data || ex;
        }
        let response;
        try {
            response = yield globals_1.axios.post(`${dtUrl}api/config/v1/extensions/${pluginId}/instances`, payload, {
                headers: {
                    Authorization: "Api-Token " + (req.header('Authorization') || globals_1.AuthConfig.altToken)
                }
            });
            return response.data;
        }
        catch (ex) {
            return {
                ex,
                data: response.data
            };
        }
    });
}
// Download and install a plugin. (Payload is json)
router.post('/stage/json', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const url = req.body.url;
    if (url.length < 10)
        return res.status(400).send("Invalid URL");
    const fileName = "download";
    const writer = fs.createWriteStream(stageDir + fileName + '.zip');
    globals_1.repoLogger.info(`Downloading '${url}'`);
    const response = yield globals_1.axios.get(url, {
        responseType: 'stream'
    });
    response.data.pipe(writer);
    // Await completion of the download.
    // BART: This looks weird. Why not try/catch?
    let msg = yield new Promise((resolve, reject) => {
        writer.on('finish', done => {
            resolve({ success: true, done });
        });
        writer.on('error', err => {
            reject({ success: false, err });
        });
    });
    if (msg.success) {
        globals_1.repoLogger.info(`Finished downloading '${url}'`);
    }
    else {
        globals_1.repoLogger.error(`Failed to download '${url}'. Stacktrace:`);
        globals_1.repoLogger.error(msg.err);
        return res.send({ msg, url });
    }
    let installResult = yield installPlugin(fileName, req);
    return res.send(installResult);
})));
// Upload and install a plugin.
router.post('/stage/zip', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    // Create an incoming form object.
    var form = new formidable_1.default.IncomingForm();
    form.multiples = true; // TBD
    form.uploadDir = stageDir;
    let uploadedFiles = [];
    // Every time a file has been uploaded successfully,
    // rename it to it's orignal name.
    form.on('file', function (field, file) {
        let newPath = path_1.default.join(form.uploadDir, file.name);
        fs.renameSync(file.path, newPath);
        uploadedFiles.push(file.name);
    });
    form.on('error', function (err) {
        console.log('An error has occured: \n' + err);
    });
    form.on('end', function () {
        return __awaiter(this, void 0, void 0, function* () {
            console.log("Installing ", uploadedFiles[0]);
            let installResult = yield installPlugin(uploadedFiles[0].replace(/\.zip$/, ''), req);
            res.send(installResult);
        });
    });
    // Parse the incoming request containing the form data.
    // This will start the reading of the uploaded form data which
    // will call the registered handlers above.
    form.parse(req);
})));
// Move the plugin into the correct directory (custom.python.remote.esa-v1-instance)
router.post('/unstage', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const arg = yield unstageAndInstall(req.body, req);
        res.send(arg);
    }
    catch (err) {
        next(err);
    }
})));
router.post('/delete/:id', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const id = req.params.id;
    // Validate that the referenced plugin exists where we expect it. This should help prevent attacks.
    const module = globals_1.solutionFinder.findSolutionById(id) || globals_1.solutionFinder.findExtensionById(id);
    if (!module)
        return res.status(400).send("Solution was not found");
    globals_1.repoLogger.warn("Uninstalling    " + JSON.stringify(module, null, 2));
    const dir = path_1.default.normalize(__dirname + '/../../../' + module.config.id);
    // Purge it from PM2.
    globals_1.repoLogger.warn("    Terminating PM2 processes...");
    yield globals_1.solutionFinder.removeSolution(module);
    // TODO: Add removal for "Node module mode".
    globals_1.repoLogger.warn("    Done terminating PM2 processes");
    // After it's purged, empty the directory.
    globals_1.repoLogger.warn("    Purging directory...");
    shelljs_1.default.rm('-rf', dir);
    globals_1.repoLogger.warn("    Done purging directory");
    res.send({ status: "OK" });
})));
module.exports = router;
