"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const utils_1 = require("../utils/utils");
const shelljs_1 = __importDefault(require("shelljs"));
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const authorizer_1 = require("@dt-esa/authorizer");
const globals_1 = require("../globals");
let router = express_1.default.Router();
router.get('/:id', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const id = req.params.id;
    // Validate that the referenced plugin exists where we expect it. This should help prevent attacks.
    let solution = globals_1.solutionFinder.findSolutionById(id);
    if (!solution) {
        yield globals_1.solutionFinder.discover();
        solution = globals_1.solutionFinder.findSolutionById(id);
        if (!solution)
            return res.status(400).send("Solution not found");
    }
    const dir = path_1.default.normalize(__dirname + '/../../../' + solution.config.id + '/config');
    if (!fs.existsSync(dir))
        return res.status(200).send({ message: "Directory not found" });
    const allFiles = shelljs_1.default.ls(dir);
    const schemaFiles = allFiles
        .filter(f => f.includes('.schema.'));
    const rawFiles = allFiles
        // Supported file extensions.
        .filter(f => /\.(json|yaml|yml|conf|ini|xml)$/i.test(f))
        // Exclude 'schema' and 'default' files.
        .filter(f => !f.includes('.schema.') && !f.includes('.default.'))
        // Exclude config files for which we have a schema.
        .filter(f => !schemaFiles.find(s => f === s.replace('.schema.', '.')));
    res.send({
        schemaFiles,
        rawFiles
    });
})));
router.get('/:id/:file', (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const id = req.params.id;
    const file = req.params.file;
    const schema = req.query.schema;
    let solution = globals_1.solutionFinder.findSolutionById(id);
    if (!solution) {
        yield globals_1.solutionFinder.discover();
        solution = globals_1.solutionFinder.findSolutionById(id);
        if (!solution)
            return res.status(400).send("Solution not found");
    }
    const dir = path_1.default.normalize(__dirname + '/../../../' + solution.config.id + '/config');
    if (!fs.existsSync(dir))
        return res.status(200).send({ message: "Directory not found" });
    const targetFile = dir + '/' + file;
    const configFile = targetFile.replace('.schema.', '.');
    // If a schema is requested, return it and the actual configuration.
    if (schema) {
        res.send({
            schema: fs.existsSync(targetFile) ? JSON.parse(fs.readFileSync(targetFile, { encoding: 'utf8' }).trim()) : null,
            config: fs.existsSync(configFile) ? JSON.parse(fs.readFileSync(configFile, { encoding: 'utf8' }).trim()) : null
        });
    }
    else {
        res.send({
            data: fs.existsSync(targetFile) ? fs.readFileSync(targetFile, { encoding: 'utf8' }).trim() : ""
        });
    }
})));
// Save the settings for the specified plugin.
router.post('/:id/:file', (0, authorizer_1.authorize)(['WriteConfig']), (0, utils_1.route)((req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const log = globals_1.repoLogger;
    const id = req.params.id;
    const file = req.params.file;
    let solution = globals_1.solutionFinder.findSolutionById(id);
    if (!solution) {
        yield globals_1.solutionFinder.discover();
        solution = globals_1.solutionFinder.findSolutionById(id);
        if (!solution)
            return res.status(400).send("Solution not found");
    }
    const dir = path_1.default.normalize(__dirname + '/../../../' + solution.config.id + '/config');
    const configFile = dir + '/' + file.replace('.schema.', '.');
    log.info("Updating configuration file:");
    log.info(JSON.stringify(configFile, null, 4));
    fs.writeFileSync(configFile, typeof req.body.data == "object" ? JSON.stringify(req.body.data, null, 2) : req.body.data);
    res.send({ message: 'OK' });
})));
module.exports = router;
