"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const https_1 = __importDefault(require("https"));
const express_1 = __importDefault(require("express"));
const chalk_1 = __importDefault(require("chalk"));
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const morgan_1 = __importDefault(require("morgan"));
const compression_1 = __importDefault(require("compression"));
const express_http_proxy_1 = __importDefault(require("express-http-proxy"));
const helmet_1 = __importDefault(require("helmet"));
const selfsigned_1 = __importDefault(require("selfsigned"));
const express_session_1 = __importDefault(require("express-session"));
const authorizer_1 = require("@dt-esa/authorizer");
const errorhandler_1 = require("./utils/errorhandler");
const globals_1 = require("./globals");
// Devmode chaos to make sure the workers can go down safely without completely disrupting service.
// Also helps reduce the event loop pollution when using an inspector.
if (globals_1.devMode) {
    const runningInPM2 = /[\\\/]pm2[\\\/]/.test(process.argv[1]);
    // When using PM2, we want to auto restart workers.
    if (runningInPM2)
        setTimeout(() => {
            console.warn("Development mode auto restart. This should NOT interrupt service.");
            process.exit();
        }, (Math.round(Math.random() * 10) % 3 - 1 + 30) * 60 * 1000);
}
console.log(chalk_1.default.blueBright("Starting Solution Server running at: "), chalk_1.default.gray(process.cwd()));
console.log(chalk_1.default.blueBright("Arguments: "), chalk_1.default.gray(process.argv.join(', ')));
// Setup base app middleware.
const app = (0, express_1.default)();
globals_1.mainLogger.info("Configuration:");
globals_1.mainLogger.info(JSON.stringify(globals_1.Config, undefined, 4));
// Enable compression. Don't compress responses when explicitly asked not to.
app.use((0, compression_1.default)({
    filter: (req, res) => req.headers['x-no-compression'] ? false : compression_1.default.filter(req, res)
}));
app.use((0, cookie_parser_1.default)());
app.use((0, express_session_1.default)({
    secret: globals_1.SessionSecret,
    resave: true,
    saveUninitialized: false
}));
app.use(express_1.default.urlencoded({ extended: true }));
app.use((0, authorizer_1.authentication)(globals_1.AuthConfig));
// Log HTTP Access.
app.use((0, morgan_1.default)((tokens, req, res) => [
    tokens.date('web'),
    `[${tokens['remote-addr'](req, res)}(${req._username || 'anonymous'})]`,
    '-',
    `(${tokens.method(req, res)}/${tokens.status(req, res)})`,
    `"${tokens.url(req, res)}"`,
    '-',
    tokens['response-time'](req, res) + 'ms',
    '-',
    (res.get("content-length") || '-') + 'b'
].join(' '), {
    skip: (req, res) => {
        return res.statusCode < 400 || req.url == "/api/heath";
    }
}));
// Enable JSON requests/responses.
app.use(express_1.default.json());
// Set proper CSP directives for browser.
app.use(helmet_1.default.contentSecurityPolicy({
    useDefaults: true,
    directives: {
        "script-src": ["'self'", "'unsafe-inline'", "'unsafe-eval'"],
        "script-src-attr": ["'self'", "'unsafe-inline'", "'unsafe-eval'"],
        "style-src": ["'self'", "'unsafe-inline'", "fonts.googleapis.com"],
        "font-src": ["'self'", "fonts.gstatic.com"],
        "img-src": ["*", "data:"]
    },
}));
// Allow DNS prefetching to make initial connections faster. 
// We don't have no PII data so whatever opts won't harm us.
app.use(helmet_1.default.dnsPrefetchControl({ allow: true }));
app.use(helmet_1.default.frameguard({ action: "sameorigin" }));
app.use(helmet_1.default.hidePoweredBy());
app.use(helmet_1.default.hsts({ maxAge: 86400 * 7 }));
app.use(helmet_1.default.permittedCrossDomainPolicies());
app.use(helmet_1.default.referrerPolicy());
app.use(helmet_1.default.xssFilter());
// Setup API routes for use with the UI.
app.use(require('./api/index'));
app.use("/api/install", (0, authorizer_1.authorize)(['ReadConfig', "WriteConfig"]), require('./api/install'));
app.use("/api/logs", (0, authorizer_1.authorize)(['logs.read']), require('./api/logs'));
app.use("/api/settings", (0, authorizer_1.authorize)(['ReadConfig']), require('./api/settings'));
app.use("/api/apps", (0, authorizer_1.authorize)(), require('./api/apps'));
// Direct requests for this path to the relevant solution.
app.use((req, res, next) => {
    const solution = globals_1.solutionFinder.findSolutionForPath(req.path);
    if (solution && solution.router)
        solution.router(req, res, next);
    else
        next();
});
if (globals_1.devMode) {
    // ! Do not worry about this in production mode.
    // Proxy ONLY KNOWN PATHS to the angular dev server.
    // This makes normally unresolved paths reject with a 404.
    app.use("/", (0, express_http_proxy_1.default)('http://127.0.0.1:4200', {
        filter: function (req, res) {
            // Only proxy requests that *should* be handled.
            // This helps us know which paths will break in a production build.
            return false ||
                req.path == '/' || // root request
                req.path.startsWith('/images') || //image reqest
                req.path.startsWith('/lib') || //lib asset reqest
                req.path.startsWith('/sockjs-node') || //lib asset reqest
                req.path.endsWith('.map') || //map file reqest
                req.path.endsWith('.woff') || //font asset reqest
                req.path.endsWith('.woff2') || //font asset reqest
                req.path == '/vendor.js' || // root request
                req.path == '/main.js' || // root request
                req.path == '/runtime.js' || // root request
                req.path == '/polyfills.js' || // root request
                req.path == '/main.js' || // root request
                req.path == '/styles.css' || // root request
                req.path == '/formio.full.min.css'; // || // root request
        },
        proxyErrorHandler: (err, res, next) => {
            next(err);
        }
    }));
}
else {
    app.use(express_1.default.static(__dirname + '/../client/'));
}
app.use((req, res, next) => next({ status: 404, message: "NotFound" }));
app.use(errorhandler_1.ErrorHandler);
// HTTPS provisions.
let sslKey = null;
let sslCert = null;
if (globals_1.Config.certs.privateKey && globals_1.Config.certs.privateKey.length >= 10) {
    sslKey = globals_1.Config.certs.privateKey;
    sslCert = globals_1.Config.certs.publicKey;
}
else if (fs.existsSync(__dirname + '/../data/generated.crt')) {
    sslKey = fs.readFileSync(__dirname + '/../data/generated.key', { encoding: 'utf8' });
    sslCert = fs.readFileSync(__dirname + '/../data/generated.crt', { encoding: 'utf8' });
}
else {
    const signedCert = selfsigned_1.default.generate([{ name: 'commonName', value: 'services.dynatrace.corp' }], {
        days: 365 * 2,
        keySize: 2048,
        algorithm: "sha512"
    });
    sslKey = signedCert.private;
    sslCert = signedCert.cert;
    fs.writeFileSync(__dirname + '/../data/generated.key', sslKey);
    fs.writeFileSync(__dirname + '/../data/generated.crt', sslCert);
}
const sOpts = {
    key: sslKey,
    cert: sslCert
};
// Create an HTTPS server that can fallback to HTTP.
const server = https_1.default.createServer(sOpts, app);
server.listen(globals_1.Config.port);
server.on('error', (error) => {
    globals_1.mainLogger.error(error);
    // BART: What does this mean?
    if (error.syscall !== 'listen')
        throw error;
    // Report friendly errors where we can.
    switch (error.code) {
        case 'EACCES':
            console.error('Port ' + globals_1.Config.port + ' requires elevated privileges');
            process.exit(1);
        case 'EADDRINUSE':
            console.error('Port ' + globals_1.Config.port + ' is already in use');
            process.exit(1);
        default:
            throw error;
    }
});
server.on('listening', () => {
    const addr = server.address();
    console.log("PID %s listening on port %s.", process.pid, addr.port || addr);
});
