"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenId = void 0;
const express_openid_connect_1 = require("express-openid-connect");
const OpenId = (router, cache, options) => {
    // Customize session store to share sessions across the process instances.
    if (!options)
        options = {};
    if (!options.session)
        options.session = {};
    options.session.store = {
        get: (id, done) => {
            cache.get(id)
                .then(res => done(null, res))
                .catch(err => done(err));
        },
        set: (id, session, done) => {
            cache.put(id, session)
                .then(res => done(null))
                .catch(err => done(err));
        },
        destroy: (id, done) => {
            cache.delete(id)
                .then(res => done(null))
                .catch(err => done(err));
        }
    };
    // This MUST BE SET.
    options.routes = {
        login: "/login/openid",
        logout: "/logout/openid",
        postLogoutRedirect: "/",
        callback: "/code/openid"
    };
    router.use((0, express_openid_connect_1.auth)(options));
    router.use((req, res, next) => {
        var _a;
        if (req.oidc && req.oidc.user)
            req._username = (_a = req.oidc.user.upn) === null || _a === void 0 ? void 0 : _a.toLowerCase();
        next();
    });
};
exports.OpenId = OpenId;
