"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorHandler = exports.renderErrorPage = exports.render404Page = exports.router = void 0;
const fs_1 = __importDefault(require("fs"));
const express_1 = __importDefault(require("express"));
const globals_1 = require("../globals");
exports.router = express_1.default.Router();
const notFoundPage = fs_1.default.readFileSync(__dirname + '/../static/404.html', { encoding: "utf8" });
const errorPage = fs_1.default.readFileSync(__dirname + '/../static/500.html', { encoding: "utf8" });
// Render the 4xx page template HTML
function render404Page(req, data) {
    const code = '404';
    const title = "Not Found";
    const message = `We can't find this page.<br/> ${req.path}`;
    let html = notFoundPage;
    // Replace the placeholders.
    html = html.replace("{{code}}", code);
    html = html.replace("{{title}}", title);
    html = html.replace("{{message}}", message);
    return html;
}
exports.render404Page = render404Page;
// Render the 5xx page template HTML
function renderErrorPage(req, err) {
    const code = '500';
    const title = err.title || err.message || "Error";
    const message = `Requested ${req.method}:${req.path}<br/>
    ${err.message || ""}`;
    let html = errorPage;
    // Replace the placeholders.
    html = html.replace("{{code}}", code);
    html = html.replace("{{title}}", title);
    html = html.replace("{{message}}", message);
    html = html.replace("{{stacktrace}}", err.stack);
    return html;
}
exports.renderErrorPage = renderErrorPage;
// Catch-all error handler.
const ErrorHandler = (err, req, res, next) => {
    const log = globals_1.mainLogger;
    // Check request 'accept' header to see if it allows HTML.
    const wantsHTML = /(text\/html|application\/xhtml\+xml)/.test(req.headers.accept);
    const error = err.stack || err.message || err.toString();
    // Don't bother logging 404 errors in the general log.
    if (err.status != 404)
        log.error(error);
    // Render the HTML page
    let html = err.status == 404 ? render404Page(req, err) : renderErrorPage(req, typeof err == 'string' ? { message: err } : err);
    // Render the error page.
    res.status(err.status || 500);
    // If the browser is accepting HTML in the request, we'll render a nice error page.
    // Otherwise: JSON.
    res.send(wantsHTML ? html : error);
};
exports.ErrorHandler = ErrorHandler;
