"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startSolution = exports.reload = exports.restart = exports.stop = exports.del = exports.describe = exports.list = void 0;
const fs = __importStar(require("fs"));
const API_js_1 = __importDefault(require("pm2/lib/API.js"));
const globals_1 = require("../globals");
const path_1 = __importDefault(require("path"));
let pm2;
let isConnected = false;
// Connect on launch and maintain a persistent connection. TODO: we should set a flag RE whether we have a PM2 connection.
connect();
// Reload connection every 5 minutes.
setInterval(() => {
    if (isConnected) {
        pm2 === null || pm2 === void 0 ? void 0 : pm2.disconnect(() => {
            isConnected = false;
            connect();
        });
        return;
    }
    connect();
}, 5 * 60 * 1000);
function connect() {
    if (!pm2) {
        try {
            pm2 = new API_js_1.default();
        }
        catch (ex) {
            console.error("Failed to connect to PM2 daemon.", ex);
        }
    }
    pm2 === null || pm2 === void 0 ? void 0 : pm2.connect((err) => {
        if (err) {
            console.error("Failed to connect to PM2 daemon.", err);
            // throw err;
        }
        else {
            isConnected = true;
            console.log("Successfully connected to PM2 daemon.");
        }
    });
}
function list() {
    globals_1.mainLogger.info("Listing all processes...");
    if (!pm2)
        throw new Error("Could not connect to PM2 daemon.");
    return new Promise((resolve, reject) => {
        pm2.Client.executeRemote('getMonitorData', {}, (err, list) => err ? reject(err) : resolve(list));
    });
}
exports.list = list;
function describe(id) {
    globals_1.mainLogger.info(`Describing [${id}]...`);
    if (!pm2)
        throw new Error("Could not connect to PM2 daemon.");
    return new Promise((resolve, reject) => list()
        .then(list => resolve(list.filter((app) => app.pm2_env.namespace == id || app.id == id)))
        .catch(reject));
}
exports.describe = describe;
function del(id) {
    globals_1.mainLogger.info(`Deleting [${id}]...`);
    if (!pm2)
        throw new Error("Could not connect to PM2 daemon.");
    return new Promise((resolve, reject) => {
        try {
            pm2.delete(id, (err, res) => err ? reject(err) : resolve(res));
        }
        catch (ex) {
            reject(ex);
        }
    });
}
exports.del = del;
function stop(id) {
    globals_1.mainLogger.info(`Stopping [${id}]...`);
    if (!pm2)
        throw new Error("Could not connect to PM2 daemon.");
    return new Promise((resolve, reject) => pm2.stop(id, (err, res) => err ? reject(err) : resolve(res)));
}
exports.stop = stop;
function restart(id) {
    globals_1.mainLogger.info(`Restarting [${id}]...`);
    if (!pm2)
        throw new Error("Could not connect to PM2 daemon.");
    return new Promise((resolve, reject) => pm2.restart(id, (err, res) => err ? reject(err) : resolve(res)));
}
exports.restart = restart;
function reload(id) {
    globals_1.mainLogger.info(`Reloading [${id}]...`);
    if (!pm2)
        throw new Error("Could not connect to PM2 daemon.");
    return new Promise((resolve, reject) => pm2.reload(id, (err, res) => err ? reject(err) : resolve(res)));
}
exports.reload = reload;
// ! Here be dragons.
// Please be aware that much of the PM2 functionality has several quirks.
// Things may not immediately break on your development system if you change something.
// If you do need to change something, make sure to test it on Linux.
function startSolution(solution) {
    return __awaiter(this, void 0, void 0, function* () {
        globals_1.mainLogger.info(`Starting solution [${solution.name}]...`);
        if (!pm2)
            throw new Error("Could not connect to PM2 daemon.");
        const sourceApps = globals_1.solutionFinder.getEcosystemApps(solution.config.id, true);
        // Destroy the data reference so we don't modify anything important.
        const ecosystemApps = JSON.parse(JSON.stringify(sourceApps));
        // Make the working directory correct relative to where we are.
        // This is also where we set the namespace, so that ecosystem
        // config writers don't have to do that 'manually'.
        ecosystemApps.forEach(app => {
            // Absolute cwd in the generated file
            app.cwd = path_1.default.normalize(app.cwd && path_1.default.isAbsolute(app.cwd)
                ? app.cwd // If the CWD provided is absolute, use that.
                : __dirname + '/../../../' + solution.config.id + '/' + (app.cwd || '')); // Otherwise, calculate the absolute CWD.
            app.namespace = solution.config.id;
            app.max_restarts = 10;
        });
        // Build location of where our temp file resides.
        const fileName = 'tmp/' + solution.name.replace(/[\///]/g, "_") + '.config.js';
        const targetFile = path_1.default.normalize(__dirname + "/../../" + fileName);
        try {
            globals_1.mainLogger.info(`    Writing solution temp file [${solution.name}]...`);
            const tempEcosystem = "module.exports = " + JSON.stringify(ecosystemApps, null, 4) + ";";
            fs.writeFileSync(targetFile, tempEcosystem);
            return new Promise((resolve, reject) => {
                globals_1.mainLogger.info(`    Calling PM2 start [${solution.name}]...`);
                pm2.start(targetFile, (err, res) => {
                    // Something failed during startup.
                    if (err) {
                        globals_1.mainLogger.error(`    ...Failed to start [${solution.name}]!`);
                        globals_1.mainLogger.error(err);
                        return reject(err);
                    }
                    // Successfully started with pm2.
                    globals_1.mainLogger.info(`    ...Started [${solution.name}].`);
                    resolve(res);
                });
            });
        }
        catch (ex) {
            globals_1.mainLogger.error(`PM2 failed to start [${solution.name}]!`);
            throw ex;
        }
    });
}
exports.startSolution = startSolution;
