"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.route = exports.getFiles = exports.getDirectories = void 0;
const fs_1 = require("fs");
// Get all directories in a directory.
const getDirectories = (source) => (0, fs_1.readdirSync)(source, { withFileTypes: true })
    .filter(dirent => dirent.isDirectory())
    .map(dirent => dirent.name);
exports.getDirectories = getDirectories;
// Get all files in a directory.
const getFiles = (source) => (0, fs_1.readdirSync)(source, { withFileTypes: true })
    .filter(dirent => !dirent.isDirectory())
    .map(dirent => dirent.name);
exports.getFiles = getFiles;
// Important
// Express does not handle failed async route handlers by default.
// We use this route to 'throw' an error when an async route has an unhandled exception.
const route = fn => (req, res, next) => {
    return Promise
        .resolve(fn(req, res, next))
        .catch(next);
};
exports.route = route;
